@@ -18,11 +18,24 @@ use Mojo::DOM;
 
 use Mojolicious::Plugin::Vparam::Address;
 
-our $VERSION    = '1.17';
+our $VERSION    = '1.18';
 
 # Shift for convert ASCII char position to simple sequence 0,1,2...9,A,B,C,,,
 our $CHAR_SHIFT = ord('A') - 10;
 
+# Regext for shortcut parser
+our $SHORTCUT_REGEXP = qr{
+    ^
+    (
+        [!?@~]                                          # symbols shortcut
+        |
+        (?:array|maybe|optional|required?|skipundef)\[  # text[] shortcut start
+    )
+    (.*?)                                               # value
+    \]?                                                 # text[] shortcut end
+    $
+}xi;
+
 =encoding utf-8
 
 =head1 NAME
@@ -943,9 +956,12 @@ Then true and value is not passed validation don`t set verrors.
     # Shortcut optional syntax
     $param2 = $self->vparam(param2 => '?int');
     $param3 = $self->vparam(param3 => 'maybe[int]');
+    $param4 = $self->vparam(param4 => 'optional[int]');
 
     # Shortcut required syntax
-    $param4 = $self->vparam(param4 => '!int');
+    $param5 = $self->vparam(param5 => '!int');
+    $param6 = $self->vparam(param6 => 'require[int]');
+    $param7 = $self->vparam(param7 => 'required[int]');
 
 =head2 skip
 
@@ -979,7 +995,7 @@ if it`s undefined by set I<skipundef>.
 
     # Simple vparam
     # myparam is undef.
-    $param6 = $self->vparam(myparam => 'int', optional => 1, skipundef => 1);
+    $param1 = $self->vparam(param1 => 'int', optional => 1, skipundef => 1);
 
     # Simple flag
     # The %params hash is empty if myparam value is not integer.
@@ -995,6 +1011,9 @@ if it`s undefined by set I<skipundef>.
         param2      => 'str',
     );
 
+    # Shortcut syntax: skipundef and optional is on
+    $param2 = $self->vparam(param2 => '~int');
+
 Arrays always return as arrayref. But undefined values will be skipped.
 
 =head2 jpath
@@ -1660,30 +1679,23 @@ sub register {
 
             # Apply type
             if( defined( my $type = $attr{type} ) ) {
+                # Parse shortcut
+                while( my ($mod, $inner) = $type =~ $SHORTCUT_REGEXP ) {
+                    last unless $inner;
+                    $type = $inner;
 
-                # Optional shortcat
-                if( my ($realtype) = $type =~ m{\?(.*?)$} ) {
-                    $type           = $realtype;
-                    $attr{optional} = 1;
-                }
-                if( my ($realtype) = $type =~ m{maybe\[(.*?)\]$}i ) {
-                    $type           = $realtype;
-                    $attr{optional} = 1;
-                }
-                # Required shortcat
-                if( my ($realtype) = $type =~ m{\!(.*?)$} ) {
-                    $type           = $realtype;
-                    $attr{optional} = 0;
-                }
-
-                # Array shortcat
-                if( my ($realtype) = $type =~ m{@(.*)$} ) {
-                    $type           = $realtype;
-                    $attr{array}    = 1;
-                }
-                if( my ($realtype) = $type =~ m{array\[(.*?)\]$}i ) {
-                    $type           = $realtype;
-                    $attr{array}    = 1;
+                    if(      $mod eq '?' || $mod =~ m{^optional\[}i) {
+                        $attr{optional} = 1;
+                    } elsif( $mod eq '!' || $mod =~ m{^required?\[}i) {
+                        $attr{optional} = 0;
+                    } elsif( $mod eq '@' || $mod =~ m{^array\[}i) {
+                        $attr{array}    = 1;
+                    } elsif(                $mod =~ m{^skipundef\[}i) {
+                        $attr{skipundef}= 1;
+                    } elsif( $mod eq '~' ) {
+                        $attr{skipundef}= 1;
+                        $attr{optional} = 1;
+                    }
                 }
 
                 if( exists $conf->{types}{ $type } ) {
